@@ -9,14 +9,13 @@ import pandas as pd
 import numpy as np
 import xlrd
 
-from pyscal import (
-    WaterOil,
-    GasOil,
-    GasWater,
-    WaterOilGas,
-    SCALrecommendation,
-    PyscalList,
-)
+from pyscal.utils import capillarypressure
+from .wateroil import WaterOil
+from .gasoil import GasOil
+from .gaswater import GasWater
+from .wateroilgas import WaterOilGas
+from .scalrecommendation import SCALrecommendation
+from .pyscallist import PyscalList
 
 logging.basicConfig()
 logger = logging.getLogger(__name__)
@@ -44,6 +43,9 @@ WO_LET_OIL = ["low", "eow", "tow"]
 WO_LET_OIL_ALT = ["lo", "eo", "to"]  # Alternative parameter names.
 WO_OIL_ENDPOINTS = ["kroend", "krowend"]  # krowend is deprecated in favour of kroend
 WO_SIMPLE_J = ["a", "b", "poro_ref", "perm_ref", "drho"]  # "g" is optional
+WO_SWLHEIGHT = ["swlheight"]
+WO_SWCR_ADD = ["swcr_add"]  # Relevant when swlheight is in use.
+WO_SWL_FROM_HEIGHT = WO_SWLHEIGHT + ["swirr", "a", "b", "poro_ref", "perm_ref"]
 WO_NORM_J = ["a", "b", "poro", "perm", "sigma_costau"]
 # 'a' in WO_NORM_J is the same as a_petro, but should possibly kept as is.
 WO_SIMPLE_J_PETRO = [
@@ -137,8 +139,7 @@ class PyscalFactory(object):
           a, b, poro, perm, sigma_costau
 
         Args:
-            params (dict): Dictionary with parameters describing
-                the WaterOil object.
+            params (dict): Dictionary with parameters describing the WaterOil object.
         """
         if not params:
             params = dict()
@@ -156,6 +157,47 @@ class PyscalFactory(object):
         params = filter_nan_from_dict(params)
 
         usedparams = set()
+
+        # Check if we should initialize swl from a swlheight parameter:
+        if set(WO_SWL_FROM_HEIGHT).issubset(params):
+            if "swl" in params:
+                raise ValueError(
+                    "Do not provide both swl and swlheight at the same time"
+                )
+            params_swl_from_height = slicedict(params, WO_SWL_FROM_HEIGHT)
+            params["swl"] = capillarypressure.swl_from_height_simpleJ(
+                **params_swl_from_height
+            )
+            if "swcr" in params and params["swcr"] < params["swl"]:
+                raise ValueError(
+                    "Provided swcr={} is lower than computed swl={}".format(
+                        params["swcr"], params["swl"]
+                    )
+                )
+        elif set(WO_SWLHEIGHT).issubset(params):
+            raise ValueError(
+                (
+                    "Can't initialize from SWLHEIGHT without "
+                    "sufficient simple-J parameters, all of: "
+                    "{}".format(WO_SWL_FROM_HEIGHT)
+                )
+            )
+
+        # Should we have a swcr relative to swl?
+        if set(WO_SWCR_ADD).issubset(params):
+            if "swl" not in params:
+                raise ValueError(
+                    (
+                        "If swcr should be relative to swl, "
+                        "both swcr_add and swl must be provided"
+                    )
+                )
+            if "swcr" in params:
+                raise ValueError(
+                    "Do not provide both swcr and swcr_add at the same time"
+                )
+            params["swcr"] = params["swl"] + params[WO_SWCR_ADD[0]]
+
         # No requirements to the base objects, defaults are ok.
         wateroil = WaterOil(**slicedict(params, WO_INIT))
         usedparams = usedparams.union(set(slicedict(params, WO_INIT).keys()))
@@ -386,6 +428,11 @@ class PyscalFactory(object):
             logger.info("No wateroil parameters. Assuming only gas-oil in wateroilgas")
             wateroil = None
 
+        # If the swl in WaterOil was initialized with swlheight,
+        # ensure that result is passed on to the GasOil object:
+        if "swl" not in params and "swlheight" in params:
+            params["swl"] = wateroil.swl
+
         if sufficient_gas_oil_params(params, failhard=False):
             gasoil = PyscalFactory.create_gas_oil(params)
         else:
