@@ -11,7 +11,8 @@ import pandas as pd
 from scipy.interpolate import PchipInterpolator
 
 import pyscal
-from pyscal import utils
+from pyscal.utils.relperm import estimate_diffjumppoint, crosspoint
+from pyscal.utils.string import df2str, comment_formatter
 from pyscal.constants import EPSILON as epsilon
 from pyscal.constants import SWINTEGERS, MAX_EXPONENT
 
@@ -291,6 +292,7 @@ class GasOil(object):
             self.table["krg"].fillna(method="bfill", inplace=True)
             self.table["krg"].clip(lower=0.0, upper=1.0, inplace=True)
             self.krgcomment = "-- krg from tabular input" + krgcomment + "\n"
+            self.sgcr = self.estimate_sgcr()
         if krogcolname in dframe:
             if not (dframe[krogcolname].diff().dropna() < epsilon).all():
                 raise ValueError("Incoming krogcolname not decreasing")
@@ -306,6 +308,7 @@ class GasOil(object):
             self.table["krog"].fillna(method="bfill", inplace=True)
             self.table["krog"].clip(lower=0.0, upper=1.0, inplace=True)
             self.krogcomment = "-- krog from tabular input" + krogcomment + "\n"
+            self.sorg = self.estimate_sorg()
         if pccolname in dframe:
             # Incoming dataframe must cover the range:
             if dframe[sgcolname].min() > self.table["sg"].min():
@@ -556,51 +559,41 @@ class GasOil(object):
         sorg is estimated by searching for a linear part in krg downwards
         from sg=1-swl. In practice it is impossible to infer sorg = 0,
         since we are limited by h, and the last segment from sg=1-swl-h
-        to sg=1-swl can always be assumed linear.
-
-        If krgend is anchored to sorg, krg data is used to infer sorg. If not,
-        krg cannot be used for this, and krog is used. sorg might be overestimated
-        when krog is used if it very close to zero before reaching sorw.
+        to sg=1-swl must always be assumed linear.
 
         If the curve is linear everywhere, sorg will be returned as sgcr + h
 
-        Args:
-            None
+        If krgend is anchored to sorg, krg data is used to infer sorg. If not,
+        krg cannot be used for this, and krog is used. sorg might be overestimated
+        when krog is used if it very close to zero before reaching sorg.
+
         Returns:
             float: The estimated sorg.
         """
         if self.krgendanchor == "sorg":
             assert "krg" in self.table
             assert self.table["krg"].sum() > 0
-            return self.table["sg"].max() - utils.estimate_diffjumppoint(
+            return self.table["sg"].max() - estimate_diffjumppoint(
                 self.table, xcol="sg", ycol="krg", side="right"
             )
         assert "krog" in self.table
         assert self.table["krog"].sum() > 0
-        return self.table["sg"].max() - utils.estimate_diffjumppoint(
+        return self.table["sg"].max() - estimate_diffjumppoint(
             self.table, xcol="sg", ycol="krog", side="right"
         )
 
-    def estimate_sgcr(self, curve="krog"):
+    def estimate_sgcr(self):
         """Estimate sgcr of the current krog data.
 
-        sgcr is estimated by searching for a linear part in krog upwards from sg=0.
-        In practice it is impossible to infer sgcr = 0, since we are limited by
-        h, and we always have to assume that the first segment is linear.
+        sgcr is the largest gas saturation for which the gas relative
+        permeability is approximately zero, where approximately zero is
+        roughly equivalent to how many digits are outputted
+        by SGOF().
 
-        If the curve is linear everywhere, sgcr will be returned as the right endpoint.
-
-        Args:
-            curve (str): Column name to use for search for linearity. Default is krog,
-                if all of that is linear, you may try krg instead.
         Returns:
             float: The estimated sgcr.
         """
-        assert curve in self.table
-        assert self.table[curve].sum() > 0
-        return utils.estimate_diffjumppoint(
-            self.table, xcol="sg", ycol=curve, side="left"
-        )
+        return self.table[self.table["krg"] < 10 * epsilon]["sg"].max()
 
     def crosspoint(self):
         """Locate and return the saturation point where krg = krog
@@ -613,7 +606,7 @@ class GasOil(object):
             float: the gas saturation where krg == krog, for relperm
                 linearly interpolated in gas saturation.
         """
-        return utils.crosspoint(self.table, "sg", "krg", "krog")
+        return crosspoint(self.table, "sg", "krg", "krog")
 
     def selfcheck(self, mode="SGOF"):
         """Check validities of the data in the table.
@@ -698,11 +691,11 @@ class GasOil(object):
             return ""
         string = ""
         if "pc" not in self.table:
-            self.table["pc"] = 0
+            self.table["pc"] = 0.0
             self.pccomment = "-- Zero capillary pressure\n"
         if header:
             string += "SGOF\n"
-        string += utils.comment_formatter(self.tag)
+        string += comment_formatter(self.tag)
         string += "-- pyscal: " + str(pyscal.__version__) + "\n"
         if dataincommentrow:
             string += self.sgcomment
@@ -720,10 +713,13 @@ class GasOil(object):
             + "PC".ljust(width)
             + "\n"
         )
-        string += utils.df2str(
+        string += df2str(
             self.table[["sg", "krg", "krog", "pc"]],
-            monotone_column="pc",
-            monotone_direction="inc",
+            monotonocity={
+                "krog": {"sign": -1, "lower": 0, "upper": 1},
+                "krg": {"sign": 1, "lower": 0, "upper": 1},
+                "pc": {"sign": 1, "allowzero": True},
+            },
         )
         string += "/\n"
         return string
@@ -735,7 +731,7 @@ class GasOil(object):
         extracted as a single function to facilitate testing."""
         if "pc" not in self.table.columns:
             # Only happens when the SLGOF function is skipped (test code)
-            self.table["pc"] = 0
+            self.table["pc"] = 0.0
         slgof = (
             self.table[
                 self.table["sg"] <= 1 - self.sorg - self.swl + 1.0 / float(SWINTEGERS)
@@ -790,11 +786,11 @@ class GasOil(object):
             return ""
         string = ""
         if "pc" not in self.table:
-            self.table["pc"] = 0
+            self.table["pc"] = 0.0
             self.pccomment = "-- Zero capillary pressure\n"
         if header:
             string += "SLGOF\n"
-        string += utils.comment_formatter(self.tag)
+        string += comment_formatter(self.tag)
         string += "-- pyscal: " + str(pyscal.__version__) + "\n"
         if dataincommentrow:
             string += self.sgcomment
@@ -811,8 +807,13 @@ class GasOil(object):
             + "PC".ljust(width)
             + "\n"
         )
-        string += utils.df2str(
-            self.slgof_df(), monotone_column="pc", monotone_direction="dec"
+        string += df2str(
+            self.slgof_df(),
+            monotonocity={
+                "krog": {"sign": 1, "lower": 0, "upper": 1},
+                "krg": {"sign": -1, "lower": 0, "upper": 1},
+                "pc": {"sign": -1, "allowzero": True},
+            },
         )
         string += "/\n"
         return string
@@ -845,11 +846,11 @@ class GasOil(object):
         """
         string = ""
         if "pc" not in self.table.columns:
-            self.table["pc"] = 0
+            self.table["pc"] = 0.0
             self.pccomment = "-- Zero capillary pressure\n"
         if header:
             string += "SGFN\n"
-        string += utils.comment_formatter(self.tag)
+        string += comment_formatter(self.tag)
         string += "-- pyscal: " + str(pyscal.__version__) + "\n"
         if dataincommentrow:
             if sgcomment is not None:
@@ -871,10 +872,12 @@ class GasOil(object):
             + "PC".ljust(width)
             + "\n"
         )
-        string += utils.df2str(
+        string += df2str(
             self.table[["sg", "krg", "pc"]],
-            monotone_column="pc",
-            monotone_direction="inc",
+            monotonocity={
+                "krg": {"sign": 1, "lower": 0, "upper": 1},
+                "pc": {"sign": 1, "allowzero": True},
+            },
         )
         string += "/\n"
         return string
@@ -899,7 +902,7 @@ class GasOil(object):
         """
         string = ""
         if "pc" not in self.table.columns:
-            self.table["pc"] = 0
+            self.table["pc"] = 0.0
             self.pccomment = "-- Zero capillary pressure\n"
         if header:
             string += "GOTABLE\n"
@@ -920,10 +923,13 @@ class GasOil(object):
             + "PC".ljust(width)
             + "\n"
         )
-        string += utils.df2str(
+        string += df2str(
             self.table[["sg", "krg", "krog", "pc"]],
-            monotone_column="pc",
-            monotone_direction="inc",
+            monotonocity={
+                "krog": {"sign": -1, "lower": 0, "upper": 1},
+                "krg": {"sign": 1, "lower": 0, "upper": 1},
+                "pc": {"sign": 1, "allowzero": True},
+            },
         )
         return string
 
@@ -944,6 +950,8 @@ class GasOil(object):
         matplotlib axis to plot on, if None, a fresh plot
         will be made.
         """
+        # pylint: disable=import-outside-toplevel
+        # Lazy import of matplotlib for speed reasons
         import matplotlib.pyplot as plt
         import matplotlib
 
