@@ -5,9 +5,10 @@ import argparse
 
 import logging
 
-from pyscal import WaterOilGas, GasWater, PyscalFactory
-
 from pyscal import __version__
+from .wateroilgas import WaterOilGas
+from .gaswater import GasWater
+from .factory import PyscalFactory
 
 logging.basicConfig()
 logger = logging.getLogger(__name__)
