@@ -13,9 +13,11 @@ import pandas as pd
 from scipy.interpolate import PchipInterpolator, interp1d
 
 import pyscal
+from pyscal.utils.relperm import estimate_diffjumppoint, crosspoint
+from pyscal.utils.string import df2str, comment_formatter
+from pyscal.utils.capillarypressure import simple_J
 from pyscal.constants import EPSILON as epsilon
 from pyscal.constants import SWINTEGERS, MAX_EXPONENT
-from pyscal import utils
 
 
 logging.basicConfig()
@@ -260,7 +262,7 @@ class WaterOil(object):
             raise ValueError("sw data not sorted")
         if krwcolname in dframe:
             if not sorw:
-                sorw = dframe[swcolname].max() - utils.estimate_diffjumppoint(
+                sorw = dframe[swcolname].max() - estimate_diffjumppoint(
                     dframe, xcol=swcolname, ycol=krwcolname, side="right"
                 )
                 logger.info("Estimated sorw in tabular data to %f", sorw)
@@ -306,10 +308,11 @@ class WaterOil(object):
             self.table["krw"].clip(lower=0.0, upper=1.0, inplace=True)
             self.sorw = sorw
             self.krwcomment = "-- krw from tabular input" + krwcomment + "\n"
+            self.swcr = self.estimate_swcr()
 
         if krowcolname in dframe:
             if not sorw:
-                sorw = dframe[swcolname].max() - utils.estimate_diffjumppoint(
+                sorw = dframe[swcolname].max() - estimate_diffjumppoint(
                     dframe, xcol=swcolname, ycol=krowcolname, side="right"
                 )
                 logger.info("Estimated sorw in tabular data from krow to %s", sorw)
@@ -395,7 +398,7 @@ class WaterOil(object):
             krwend (float): value of krw at 1 - sorw.
             krwmax (float): maximal value at Sw=1. Default 1
         """
-        assert epsilon < nw < MAX_EXPONENT
+        assert 10 * epsilon < nw < MAX_EXPONENT
         if krwmax:
             assert 0 < krwend <= krwmax <= 1.0
         else:
@@ -593,7 +596,8 @@ class WaterOil(object):
         )
 
     def add_simple_J(self, a=5, b=-1.5, poro_ref=0.25, perm_ref=100, drho=300, g=9.81):
-        """Add capillary pressure function from a simplified J-function
+        # pylint: disable=anomalous-backslash-in-string
+        r"""Add capillary pressure function from a simplified J-function
 
         This is the RMS version of the coefficients *a* and *b*, the formula
         used is
@@ -602,7 +606,7 @@ class WaterOil(object):
 
             J = a S_w^b
 
-        *J* is not dimensionless in this equation. The capillary pressure
+        *J* is not dimensionless in this equation. The capillary pressure will
         be in bars.
 
         This is identical to the also seen formula
@@ -644,15 +648,13 @@ class WaterOil(object):
                 "positive b will give increasing capillary pressure with saturation"
             )
 
-        # drho = rwo_w - rho_o, in units g/cc
-
         # swnpc is a normalized saturation, but normalized with
         # respect to swirr, not to swl (the swirr here is sometimes
         # called 'swirra' - asymptotic swirr)
-        self.table["J"] = a * self.table["swnpc"] ** b
-        self.table["H"] = self.table["J"] * math.sqrt(poro_ref / perm_ref)
-        # Scale drho and g from SI units to g/cc and m/s²100
-        self.table["pc"] = self.table["H"] * drho / 1000 * g / 100.0
+
+        self.table["pc"] = simple_J(
+            self.table["swnpc"], a, b, poro_ref, perm_ref, drho, g
+        )
         self.pccomment = (
             "-- Simplified J-function for Pc; rms version, in bar\n--   "
             + "a=%g, b=%g, poro_ref=%g, perm_ref=%g mD, drho=%g kg/m^3, g=%g m/s^2\n"
@@ -660,21 +662,22 @@ class WaterOil(object):
         )
 
     def add_simple_J_petro(self, a, b, poro_ref=0.25, perm_ref=100, drho=300, g=9.81):
-        """Add capillary pressure function from a simplified J-function
+        # pylint: disable=anomalous-backslash-in-string
+        r"""Add capillary pressure function from a simplified J-function
 
         This is the *petrophysical* version of the coefficients *a* and *b*, the formula
         used is
 
         .. math::
 
-            J = \left(\\frac{S_w}{a}\\right)^{\\frac{1}{b}}
+            J = \left(\frac{S_w}{a}\right)^{\frac{1}{b}}
 
 
         which is identical to
 
         .. math::
 
-            J = 10^\\frac{\log(S_w) - \log(a)}{b}
+            J = 10^\frac{\log(S_w) - \log(a)}{b}
 
         *J* is not dimensionless in this equation.
 
@@ -727,13 +730,14 @@ class WaterOil(object):
     def add_normalized_J(self, a, b, poro, perm, sigma_costau):
         # Don't make this a raw string to avoid the \l warning,
         # it destroys the Latex-formatting in sphinx
-        """
+        # pylint: disable=anomalous-backslash-in-string
+        r"""
         Add capillary pressure in bar through a normalized J-function.
 
         .. math::
 
-            p_c = \\frac{\left(\\frac{S_w}{a}\\right)^{\\frac{1}{b}}
-            \sigma \cos \\tau}{\sqrt{\\frac{k}{\phi}}}
+            p_c = \frac{\left(\frac{S_w}{a}\right)^{\frac{1}{b}}
+            \sigma \cos \tau}{\sqrt{\frac{k}{\phi}}}
 
         The :math:`S_w` saturation used in the formula is normalized with respect
         to the swirr parameter.
@@ -790,6 +794,7 @@ class WaterOil(object):
         )
 
     def add_skjaeveland_pc(self, cw, co, aw, ao, swr=None, sor=None):
+        # pylint: disable=line-too-long
         """Add capillary pressure from the Skjæveland correlation,
 
         Doc: https://wiki.equinor.com/wiki/index.php/Res:The_Skjaeveland_correlation_for_capillary_pressure
@@ -866,6 +871,7 @@ class WaterOil(object):
         )  # Just deduct sw to make it monotone..
 
     def add_LET_pc_pd(self, Lp, Ep, Tp, Lt, Et, Tt, Pcmax, Pct):
+        # pylint: disable=line-too-long
         """Add a primary drainage LET capillary pressure curve.
 
         Docs: https://wiki.equinor.com/wiki/index.php/Res:The_LET_correlation_for_capillary_pressure
@@ -905,6 +911,7 @@ class WaterOil(object):
         )
 
     def add_LET_pc_imb(self, Ls, Es, Ts, Lf, Ef, Tf, Pcmax, Pcmin, Pct):
+        # pylint: disable=line-too-long
         """Add an imbition LET capillary pressure curve.
 
         Docs: https://wiki.equinor.com/wiki/index.php/Res:The_LET_correlation_for_capillary_pressure
@@ -974,14 +981,14 @@ class WaterOil(object):
         """
         assert curve in self.table
         assert self.table[curve].sum() > 0
-        return self.table["sw"].max() - utils.estimate_diffjumppoint(
+        return self.table["sw"].max() - estimate_diffjumppoint(
             self.table, xcol="sw", ycol=curve, side="right"
         )
 
     def estimate_swcr(self, curve="krw"):
         """Estimate swcr of the current krw data.
 
-        swcr is estimated by searching for a linear part in krw upwards from sw=swl.
+        kwcr is estimated by searching for a linear part in krw upwards from sw=swl.
         In practice it is impossible to infer swcr = 0, since we are limited by
         h, and the first segment is assumed linear anyways.
 
@@ -997,9 +1004,7 @@ class WaterOil(object):
         """
         assert curve in self.table
         assert self.table[curve].sum() > 0
-        return utils.estimate_diffjumppoint(
-            self.table, xcol="sw", ycol=curve, side="left"
-        )
+        return estimate_diffjumppoint(self.table, xcol="sw", ycol=curve, side="left")
 
     def crosspoint(self):
         """Locate and return the saturation point where krw = krow
@@ -1011,7 +1016,7 @@ class WaterOil(object):
             float: the water saturation where krw == krow, for relperm
                 linearly interpolated in water saturation.
         """
-        return utils.crosspoint(self.table, "sw", "krw", "krow")
+        return crosspoint(self.table, "sw", "krw", "krow")
 
     def selfcheck(self, mode="SWOF"):
         """Check validities of the data in the table.
@@ -1097,10 +1102,10 @@ class WaterOil(object):
         string = ""
         if header:
             string += "SWOF\n"
-        string += utils.comment_formatter(self.tag)
+        string += comment_formatter(self.tag)
         string += "-- pyscal: " + str(pyscal.__version__) + "\n"
         if "pc" not in self.table.columns:
-            self.table["pc"] = 0
+            self.table["pc"] = 0.0
             self.pccomment = "-- Zero capillary pressure\n"
         if dataincommentrow:
             string += self.swcomment
@@ -1118,10 +1123,13 @@ class WaterOil(object):
             + "PC".ljust(width)
             + "\n"
         )
-        string += utils.df2str(
+        string += df2str(
             self.table[["sw", "krw", "krow", "pc"]],
-            monotone_column="pc",
-            monotone_direction="dec",
+            monotonocity={
+                "krow": {"sign": -1, "lower": 0, "upper": 1},
+                "krw": {"sign": 1, "lower": 0, "upper": 1},
+                "pc": {"sign": -1, "allowzero": True},
+            },
         )
         string += "/\n"  # Empty line at the end
         return string
@@ -1155,11 +1163,11 @@ class WaterOil(object):
             return ""
         string = ""
         if "pc" not in self.table.columns:
-            self.table["pc"] = 0
+            self.table["pc"] = 0.0
             self.pccomment = "-- Zero capillary pressure\n"
         if header:
             string += "SWFN\n"
-        string += utils.comment_formatter(self.tag)
+        string += comment_formatter(self.tag)
         string += "-- pyscal: " + str(pyscal.__version__) + "\n"
         if dataincommentrow:
             if swcomment is not None:
@@ -1181,10 +1189,12 @@ class WaterOil(object):
             + "PC".ljust(width)
             + "\n"
         )
-        string += utils.df2str(
+        string += df2str(
             self.table[["sw", "krw", "pc"]],
-            monotone_column="pc",
-            monotone_direction="dec",
+            monotonocity={
+                "krw": {"sign": 1, "lower": 0, "upper": 1},
+                "pc": {"sign": -1, "allowzero": True},
+            },
         )
         string += "/\n"  # Empty line at the end
         return string
@@ -1193,7 +1203,7 @@ class WaterOil(object):
         """Return a string for a Nexus WOTABLE"""
         string = ""
         if "pc" not in self.table.columns:
-            self.table["pc"] = 0
+            self.table["pc"] = 0.0
             self.pccomment = "-- Zero capillary pressure\n"
 
         if header:
@@ -1216,10 +1226,13 @@ class WaterOil(object):
             + "PC".ljust(width)
             + "\n"
         )
-        string += utils.df2str(
+        string += df2str(
             self.table[["sw", "krw", "krow", "pc"]],
-            monotone_column="pc",
-            monotone_direction="dec",
+            monotonocity={
+                "krow": {"sign": -1, "lower": 0, "upper": 1},
+                "krw": {"sign": 1, "lower": 0, "upper": 1},
+                "pc": {"sign": -1, "allowzero": True},
+            },
         )
         return string
 
@@ -1238,6 +1251,8 @@ class WaterOil(object):
         If mpl_ax is supplied, the curve will be drawn on
         that, if not, a new axis (plot) will be made
         """
+        # pylint: disable=import-outside-toplevel
+        # Lazy import for speed reaons.
         import matplotlib.pyplot as plt
         import matplotlib
 
@@ -1279,6 +1294,8 @@ class WaterOil(object):
         If the argument 'mpl_ax' is not supplied, a new plot
         window will be made. If supplied, it will draw on
         the specified axis."""
+        # pylint: disable=import-outside-toplevel
+        # Lazy import for speed reaons.
         import matplotlib.pyplot as plt
         import matplotlib
 
