@@ -12,7 +12,7 @@ import hypothesis.strategies as st
 from pyscal import GasWater
 from pyscal.constants import SWINTEGERS
 
-from common import (
+from pyscal.utils.testing import (
     check_table,
     float_df_checker,
     sat_table_str_ok,
@@ -50,6 +50,7 @@ def check_endpoints(gaswater, krwend, krwmax, krgend):
         assert np.isclose(gaswater.wateroil.table["krw"].max(), krwend)
 
 
+@settings(deadline=400)
 @given(st.text())
 def test_gaswater_tag(tag):
     """Test that we are unlikely to crash Eclipse
