@@ -11,8 +11,7 @@ import hypothesis.strategies as st
 
 from pyscal import SCALrecommendation, PyscalFactory, GasWater, WaterOilGas
 from pyscal.factory import slicedict
-
-from common import sat_table_str_ok, check_table
+from pyscal.utils.testing import sat_table_str_ok, check_table
 
 
 # Example SCAL recommendation, low case
@@ -259,7 +258,7 @@ def test_make_scalrecommendation_go():
     assert not interpolant.SWOF()
 
 
-@settings(max_examples=10, deadline=1500)
+@settings(max_examples=10, deadline=2000)
 @given(
     st.floats(min_value=-1.1, max_value=1.1), st.floats(min_value=-1.1, max_value=1.1)
 )
@@ -405,9 +404,9 @@ def test_gaswater_scal():
     dframe = pd.DataFrame(
         columns=["SATNUM", "CASE", "NW", "NG", "TAG"],
         data=[
-            [1, "pess", 2, 2, "thetag"],
-            [1, "base", 3, 3, "thetag"],
-            [1, "opt", 4, 4, "thetag"],
+            [1, "pess", 2, 2, "sometag"],
+            [1, "base", 3, 3, "sometag"],
+            [1, "opt", 4, 4, "sometag"],
         ],
     )
     rec_list = PyscalFactory.create_scal_recommendation_list(
